package com.sleepycat.je.recovery;

import com.sleepycat.je.CheckpointConfig;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.config.LongConfigParam;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.log.CheckpointMonitor;

/**
 * without this feature the checkpointer is called very 20,000,000 bytes. this
 * feature lets you configurize this.
 */
public privileged aspect CustomizeCheckpointerBytesInterval {
	/*
	 * Checkpointer
	 */
	public static final LongConfigParam EnvironmentParams.CHECKPOINTER_BYTES_INTERVAL = new LongConfigParam(
			"je.checkpointer.bytesInterval",
			new Long(0), // min
			new Long(Long.MAX_VALUE), // max
			new Long(Checkpointer.BYTES_INTERVAL), // default
			false, // mutable
			"# Ask the checkpointer to run every time we write this many bytes\n"
					+ "# to the log. If set, supercedes je.checkpointer.wakeupInterval. To\n"
					+ "# use time based checkpointing, set this to 0.");

	after(CheckpointMonitor cpm) throws DatabaseException :
		execution(CheckpointMonitor.new(EnvironmentImpl)) && this(cpm) {

		cpm.periodInBytes = cpm.envImpl.getConfigManager().getLong(
				EnvironmentParams.CHECKPOINTER_BYTES_INTERVAL);

		/*
		 * The period is reset each activation and is not synchronized with the
		 * interval counted by the Checkpointer. Use a small enough period to
		 * invoke the Checkpointer within 10% of its interval.
		 */
		cpm.periodInBytes /= 10;
	}

	long around(CheckpointConfig config, Checkpointer checkpointer)
			throws DatabaseException:
		execution(long Checkpointer.getBytesInterval(CheckpointConfig)) && 
		args(config) && this(checkpointer){

		long result = proceed(config, checkpointer);
		if (result == Checkpointer.BYTES_INTERVAL)
			if (config.getKBytes() << 10 != Checkpointer.BYTES_INTERVAL) {
				return checkpointer.getEnv().getConfigManager().getLong(
						EnvironmentParams.CHECKPOINTER_BYTES_INTERVAL);
			}
		return result;
	}

	long around(CheckpointConfig config) throws DatabaseException :
		execution(long Checkpointer.getBytesInterval(CheckpointConfig)) && args(config){
		if (config.getKBytes() != 0) {
			return config.getKBytes() << 10;
		} else {
			return proceed(config);
		}
	}

	private int CheckpointConfig.kbytes = 0;

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public void CheckpointConfig.setKBytes(int kbytes) {
		this.kbytes = kbytes;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int CheckpointConfig.getKBytes() {
		return kbytes;
	}
}
